#ifndef RK_DNSPACK_BRAVAISLATTICE_H
#define RK_DNSPACK_BRAVAISLATTICE_H

#include <stdexcept>

#include "CartesianCoordinate.h"

namespace rk
{
	namespace dnspack
	{
		class BravaisLattice
		{
		public:
			BravaisLattice() noexcept;
			virtual ~BravaisLattice();

		public:
			BravaisLattice(const BravaisLattice& other) noexcept;
			BravaisLattice(BravaisLattice&& other) noexcept;
			BravaisLattice& operator=(const BravaisLattice& other) noexcept;
			BravaisLattice& operator=(BravaisLattice&& other) noexcept;

			bool operator==(const BravaisLattice& other) const noexcept;
			bool operator!=(const BravaisLattice& other) const noexcept;

		public:
			const CartesianCoordinate& getLatticeVector(int label) const;
			CartesianCoordinate& getLatticeVector(int label);

			const CartesianCoordinate& operator[](int label) const noexcept;
			CartesianCoordinate& operator[](int label) noexcept;

			double getVolume() const noexcept;

		private:
			CartesianCoordinate m_latticeVectors[3];
		};
	}
}

inline rk::dnspack::BravaisLattice& rk::dnspack::BravaisLattice::operator=(const BravaisLattice& other) noexcept
{
	m_latticeVectors[0] = other.m_latticeVectors[0];
	m_latticeVectors[1] = other.m_latticeVectors[1];
	m_latticeVectors[2] = other.m_latticeVectors[2];

	return *this;
}

inline rk::dnspack::BravaisLattice& rk::dnspack::BravaisLattice::operator=(BravaisLattice&& other) noexcept
{
	m_latticeVectors[0] = std::move(other.m_latticeVectors[0]);
	m_latticeVectors[1] = std::move(other.m_latticeVectors[1]);
	m_latticeVectors[2] = std::move(other.m_latticeVectors[2]);

	return *this;
}

inline bool rk::dnspack::BravaisLattice::operator==(const rk::dnspack::BravaisLattice& other) const noexcept
{
	for (int i = 0; i < 3; ++i)
	{
		if (m_latticeVectors[i] != other.m_latticeVectors[i])
		{
			return false;
		}
	}

	return true;
}

inline bool rk::dnspack::BravaisLattice::operator!=(const rk::dnspack::BravaisLattice& other) const noexcept
{
	return !(*this == other);
}

inline const rk::dnspack::CartesianCoordinate& rk::dnspack::BravaisLattice::getLatticeVector(int label) const
{
	if (label < 0 || 2 < label)
	{
		throw std::invalid_argument("BravaisLattice::getLatticeVector(int label) -> the label is unsuitable.");
	}

	return m_latticeVectors[label];
}

inline rk::dnspack::CartesianCoordinate& rk::dnspack::BravaisLattice::getLatticeVector(int label)
{
	if (label < 0 || 2 < label)
	{
		throw std::invalid_argument("BravaisLattice::getLatticeVector(int label) -> the label is unsuitable.");
	}

	return m_latticeVectors[label];
}

inline const rk::dnspack::CartesianCoordinate& rk::dnspack::BravaisLattice::operator[](int label) const noexcept
{
	return m_latticeVectors[label];
}

inline rk::dnspack::CartesianCoordinate& rk::dnspack::BravaisLattice::operator[](int label) noexcept
{
	return m_latticeVectors[label];
}


#endif // !RK_DNSPACK_BRAVAISLATTICE_H
