#ifndef RK_DNSPACK_CALCULATIONANALYSIS_H
#define RK_DNSPACK_CALCULATIONANALYSIS_H

#include <string>
#include <mutex>
#include <stdexcept>

namespace rk
{
	namespace dnspack
	{
		class RejectNumbers
		{
		public:
			RejectNumbers();
			virtual ~RejectNumbers();

		public:
			int initialArrangeMinusVolumeRejectNumber;
			int spherePackingOptimizeMinusVolumeRejectNumber;
			int spherePackingOptimizeOverlapRejectNumber;
			int spherePackingOptimizeZvalueRejectNumber;

		public:
			RejectNumbers(const RejectNumbers&) = default;
			RejectNumbers(RejectNumbers&&) noexcept = default;
			RejectNumbers& operator=(const RejectNumbers&) = default;
			RejectNumbers& operator=(RejectNumbers&&) noexcept = default;
		};

		class CalculationAnalysis
		{
		public:
			CalculationAnalysis();
			virtual ~CalculationAnalysis();

		public:
			void appendRejectNumber(const RejectNumbers& rejectNumbers) noexcept;
			void appendRejectNumber(const RejectNumbers& rejectNumbers, std::mutex& reportMutex) noexcept;
			void clear() noexcept;

			const RejectNumbers& getRejectNumbers() const noexcept;

		public:
			static int fillingFactorDistribution[25];

		private:
			static RejectNumbers m_rejectNumbers;

		private:
			CalculationAnalysis(const CalculationAnalysis&) = delete;
			CalculationAnalysis(CalculationAnalysis&&) noexcept = delete;
			CalculationAnalysis& operator=(const CalculationAnalysis&) = delete;
			CalculationAnalysis& operator=(CalculationAnalysis&&) noexcept = delete;
		};
	}
}


inline const rk::dnspack::RejectNumbers& rk::dnspack::CalculationAnalysis::getRejectNumbers() const noexcept
{
	return m_rejectNumbers;
}

inline void rk::dnspack::CalculationAnalysis::appendRejectNumber(const RejectNumbers& rejectNumbers) noexcept
{
	m_rejectNumbers.initialArrangeMinusVolumeRejectNumber += rejectNumbers.initialArrangeMinusVolumeRejectNumber;
	m_rejectNumbers.spherePackingOptimizeMinusVolumeRejectNumber += rejectNumbers.spherePackingOptimizeMinusVolumeRejectNumber;
	m_rejectNumbers.spherePackingOptimizeOverlapRejectNumber += rejectNumbers.spherePackingOptimizeOverlapRejectNumber;
	m_rejectNumbers.spherePackingOptimizeZvalueRejectNumber += rejectNumbers.spherePackingOptimizeZvalueRejectNumber;
}

inline void rk::dnspack::CalculationAnalysis::appendRejectNumber(const RejectNumbers& rejectNumbers, std::mutex& reportMutex) noexcept
{
	std::lock_guard<std::mutex> lock{ reportMutex };

	m_rejectNumbers.initialArrangeMinusVolumeRejectNumber += rejectNumbers.initialArrangeMinusVolumeRejectNumber;
	m_rejectNumbers.spherePackingOptimizeMinusVolumeRejectNumber += rejectNumbers.spherePackingOptimizeMinusVolumeRejectNumber;
	m_rejectNumbers.spherePackingOptimizeOverlapRejectNumber += rejectNumbers.spherePackingOptimizeOverlapRejectNumber;
	m_rejectNumbers.spherePackingOptimizeZvalueRejectNumber += rejectNumbers.spherePackingOptimizeZvalueRejectNumber;
}

inline void rk::dnspack::CalculationAnalysis::clear() noexcept
{
	m_rejectNumbers.initialArrangeMinusVolumeRejectNumber = 0;
	m_rejectNumbers.spherePackingOptimizeMinusVolumeRejectNumber = 0;
	m_rejectNumbers.spherePackingOptimizeOverlapRejectNumber = 0;
	m_rejectNumbers.spherePackingOptimizeZvalueRejectNumber = 0;

	for (int label = 0; label < 25; ++label)
	{
		fillingFactorDistribution[label] = 0;
	}
}


#endif // !RK_DNSPACK_CALCULATIONANALYSIS_H
