#ifndef RK_DNSPACK_EXPLOREMANAGER_H
#define RK_DNSPACK_EXPLOREMANAGER_H

#include <string>

#include "BasisComposition.h"
#include "PackingFactory.h"

namespace rk
{
	namespace dnspack
	{
		class ExploreManager
		{
		public:
			class ExploreParameters
			{
			public:
				std::string outputDirectoryName;

				int thresholdTrialNumber;
				int maxTrialNumber;

				double initialExpandMaxChangeSize;
				int initialExpandMaxIteratorNumber;
				double initialArrangeMaxChangeSizeUpper;
				double initialArrangeMaxChangeSizeLower;
				int initialArrangeIteratorNumber;
				double spherePackingOptimizeMaxChangeSizeUpper;
				double spherePackingOptimizeMaxChangeSizeLower;
				double spherePackingOptimizeDecreaseFactor;
				int spherePackingOptimizeIteratorNumber;

				int threadNumber;
				double outputMinimumFillingFactor;

			public:
				ExploreParameters() = default;
				virtual ~ExploreParameters() {}
				ExploreParameters(const ExploreParameters&) = default;
				ExploreParameters(ExploreParameters&&) noexcept = default;
				ExploreParameters& operator=(const ExploreParameters&) = default;
				ExploreParameters& operator=(ExploreParameters&&) noexcept = default;
			};

		public:
			ExploreManager(const BasisComposition& basisComposition, const ExploreManager::ExploreParameters& exploreParameters);
			virtual ~ExploreManager();

		public:
			int execute();

		private:
			const BasisComposition m_basisComposition;
			const ExploreParameters m_exploreParameters;

		private:
			ExploreManager(const ExploreManager&) = delete;
			ExploreManager(ExploreManager&&) noexcept = delete;
			ExploreManager& operator=(const ExploreManager&) = delete;
			ExploreManager& operator=(ExploreManager&&) noexcept = delete;
		};
	}
}


#endif // !RK_DNSPACK_EXPLOREMANAGER_H
