#ifndef RK_DNSPACK_ARRANGESTRUCTURE_H
#define RK_DNSPACK_ARRANGESTRUCTURE_H

#include <valarray>

#include "SphereLabel.h"
#include "PackingStructure.h"

namespace rk
{
	namespace dnspack
	{
		class InitialArrange
		{
		public:
			InitialArrange(PackingStructure& packingStructure, const int maxStepNumber, const double maxChangeSize);
			virtual ~InitialArrange();

		public:
			bool operator()(); // volumeɂȂꍇfalseԂ

		private:
			bool setNeighborSpheres();
			void calculateGradient();
			void optimize();

			double potential(const double z) const noexcept;
			double force(const double z) const noexcept;

		private:
			PackingStructure& m_packingStructure;

			const int m_spheresNumber;
			const int m_maxStepNumber;

			std::vector<std::pair<int, SphereLabel>> m_neighborSpheres{}; // baseIndex <= sphereIndex̏ꍇ̂
			std::valarray<double> m_gradientVector{};

			double m_maxChangeSize;
			const double m_pressure{ 0.1 };

		private:
			InitialArrange(const InitialArrange&) = delete;
			InitialArrange(InitialArrange&&) noexcept = delete;
			InitialArrange& operator=(const InitialArrange&) = delete;
			InitialArrange& operator=(InitialArrange&&) noexcept = delete;
		};
	}
}


inline bool rk::dnspack::InitialArrange::operator()()
{
	for (int rep = 0; rep < 20; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < 10; ++step)
			{
				calculateGradient();
				optimize();
			}
		}

		else
		{
			return false;
		}
	}

	if (m_packingStructure.getBravaisLattice().getVolume() < 0)
	{
		return false;
	}

	int repNumber = m_maxStepNumber / 100;
	int repSurplus = m_maxStepNumber % 100;

	for (int rep = 0; rep < repNumber; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < 100; ++step)
			{
				calculateGradient();
				optimize();
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < repSurplus; ++step)
			{
				calculateGradient();
				optimize();
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	return true;
}

inline double rk::dnspack::InitialArrange::potential(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0 * z);
	}

	else
	{
		return (0);
	}
}

inline double rk::dnspack::InitialArrange::force(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0);
	}

	else
	{
		return (0);
	}
}


#endif // !RK_DNSPACK_ARRANGESTRUCTURE_H
