#ifndef RK_DNSPACK_INITIALEXPAND_H
#define RK_DNSPACK_INITIALEXPAND_H

#include <valarray>

#include "SphereLabel.h"
#include "PackingStructure.h"


namespace rk
{
	namespace dnspack
	{
		class InitialExpand
		{
		public:
			InitialExpand(PackingStructure& packingStructure, const int maxStepNumber, const double maxChangeSize);
			virtual ~InitialExpand();

		public:
			void operator()();

		private:
			void setNeighborSpheres();
			void calculateGradient();
			bool optimize();

			double potential(const double z) const noexcept;
			double force(const double z) const noexcept;

		private:
			PackingStructure& m_packingStructure;

			const int m_spheresNumber;
			const int m_maxStepNumber;

			std::vector<std::pair<int, SphereLabel>> m_neighborSpheres{}; // baseIndex <= sphereIndex̏ꍇ̂
			std::valarray<double> m_gradientVector{};

			double m_maxChangeSize;

		private:
			InitialExpand(const InitialExpand&) = delete;
			InitialExpand(InitialExpand&&) noexcept = delete;
			InitialExpand& operator=(const InitialExpand&) = delete;
			InitialExpand& operator=(InitialExpand&&) noexcept = delete;
		};
	}
}


inline void rk::dnspack::InitialExpand::operator()()
{
	setNeighborSpheres();

	for (int step = 0; step < m_maxStepNumber; ++step)
	{
		calculateGradient();

		if (optimize())
		{
			break;
		}
	}
}

inline double rk::dnspack::InitialExpand::potential(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0 * z);
	}

	else
	{
		return (0);
	}
}

inline double rk::dnspack::InitialExpand::force(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0);
	}

	else
	{
		return (0);
	}
}


#endif // !RK_DNSPACK_INITIALEXPAND_H
