#ifndef RK_DNSPACK_MAKEDATFILE_H
#define RK_DNSPACK_MAKEDATFILE_H

#include <string>
#include <sstream>
#include <fstream>
#include <mutex>
#include <stdexcept>

#include "PackingStructure.h"

namespace rk
{
	namespace dnspack
	{
		class MakeDatFile
		{
		public:
			MakeDatFile(const std::string& fileName, PackingStructure& packingStructure);
			virtual ~MakeDatFile();

		public:
			const std::string& getFileName() const noexcept;

			void writePackingInfo();
			void appendMessage(const std::string& message) noexcept;
			void makeFile();
			void makeFile(std::mutex& outputMutex);

			void clear() noexcept;

		private:
			const std::string m_fileName;
			std::stringstream m_textStream;

			PackingStructure& m_packingStructure;
			BasisComposition m_basisComposition;

		private:
			MakeDatFile(const MakeDatFile& other) = delete;
			MakeDatFile(MakeDatFile&& other) noexcept = delete;
			MakeDatFile& operator=(const MakeDatFile& other) = delete;
			MakeDatFile& operator=(MakeDatFile&& other) noexcept = delete;
		};
	}
}

inline const std::string& rk::dnspack::MakeDatFile::getFileName() const noexcept
{
	return m_fileName;
}

inline void rk::dnspack::MakeDatFile::appendMessage(const std::string& message) noexcept
{
	m_textStream << message << "\n";
}

inline void rk::dnspack::MakeDatFile::makeFile()
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open dat file.");
	}

	outputFileStream << m_textStream.str();
	outputFileStream.close();
}

inline void rk::dnspack::MakeDatFile::makeFile(std::mutex& outputMutex)
{
	std::lock_guard<std::mutex> lock{ outputMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open dat file.");
	}

	outputFileStream << m_textStream.str();
	outputFileStream.close();
}

inline void rk::dnspack::MakeDatFile::clear() noexcept
{
	m_textStream.str("");
}


#endif // !RK_DNSPACK_MAKEDATFILE_H
