#ifndef RK_DNSPACK_MAKERESULTFILE_H
#define RK_DNSPACK_MAKERESULTFILE_H

#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <mutex>

namespace rk
{
	namespace dnspack
	{
		class MakeResultFile
		{
		public:
			MakeResultFile();
			virtual ~MakeResultFile();

		public:
			static void setResultFileName(const std::string& resultFileName);
			void reportMessage(const std::string& message);
			void reportMessage(const std::string& message, std::mutex& outputMutex);
			void clear();

		private:
			static std::string m_fileName;

		private:
			MakeResultFile(const MakeResultFile& other) = delete;
			MakeResultFile(MakeResultFile&& other) noexcept = delete;
			MakeResultFile& operator=(const MakeResultFile& other) = delete;
			MakeResultFile& operator=(MakeResultFile&& other) noexcept = delete;
		};
	}
}

inline void rk::dnspack::MakeResultFile::setResultFileName(const std::string& resultFileName)
{
	m_fileName = resultFileName;

	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open current situation file.");
	}

	outputFileStream.close();
}

inline void rk::dnspack::MakeResultFile::clear()
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open current situation file.");
	}

	outputFileStream.close();
}

inline void rk::dnspack::MakeResultFile::reportMessage(const std::string& message)
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::app);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open result file.");
	}

	outputFileStream << message;
	outputFileStream.close();

	std::cout << message;
}

inline void rk::dnspack::MakeResultFile::reportMessage(const std::string& message, std::mutex& outputMutex)
{
	std::lock_guard<std::mutex> lock{ outputMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::app);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open result file.");
	}

	outputFileStream << message;
	outputFileStream.close();

	std::cout << message;
}


#endif // !RK_DNSPACK_MAKERESULTFILE_H
