#ifndef RK_DNSPACK_MAKEXSFFILE_H
#define RK_DNSPACK_MAKEXSFFILE_H

#include <string>
#include <sstream>
#include <fstream>
#include <mutex>

#include "PackingStructure.h"

namespace rk
{
	namespace dnspack
	{
		class MakeXsfFile
		{
		public:
			MakeXsfFile(const std::string& fileName, PackingStructure& packingStructure);
			virtual ~MakeXsfFile();

		public:
			const std::string& getFileName() const noexcept;

			void writePackingInfo();
			void makeFile();
			void makeFile(std::mutex& outputMutex);

		private:
			const std::string m_fileName;
			std::stringstream m_textStream;

			PackingStructure& m_packingStructure;
			BasisComposition m_basisComposition;

		private:
			MakeXsfFile(const MakeXsfFile& other) = delete;
			MakeXsfFile(MakeXsfFile&& other) noexcept = delete;
			MakeXsfFile& operator=(const MakeXsfFile& other) = delete;
			MakeXsfFile& operator=(MakeXsfFile&& other) noexcept = delete;
		};
	}
}

inline const std::string& rk::dnspack::MakeXsfFile::getFileName() const noexcept
{
	return m_fileName;
}

inline void rk::dnspack::MakeXsfFile::makeFile()
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open dat file.");
	}

	outputFileStream << m_textStream.str();
	outputFileStream.close();
}

inline void rk::dnspack::MakeXsfFile::makeFile(std::mutex& outputMutex)
{
	std::lock_guard<std::mutex> lock{ outputMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open dat file.");
	}

	outputFileStream << m_textStream.str();
	outputFileStream.close();
}

#endif // !RK_DNSPACK_MAKEDATFILE_H
