#ifndef RK_DNSPACK_PACKINGFACTORY_H
#define RK_DNSPACK_PACKINGFACTORY_H

#include <mutex>
#include <string>

#include "BasisComposition.h"
#include "PackingStructure.h"
#include "CalculationAnalysis.h"

namespace rk
{
	namespace dnspack
	{
		class PackingFactory
		{
		public:
			class OptimizeParameters
			{
			public:
				double initialExpandMaxChangeSize;
				int initialExpandMaxIteratorNumber;
				double initialArrangeMaxChangeSizeUpper;
				double initialArrangeMaxChangeSizeLower;
				int initialArrangeIteratorNumber;
				double spherePackingOptimizeMaxChangeSizeUpper;
				double spherePackingOptimizeMaxChangeSizeLower;
				double spherePackingOptimizeDecreaseFactor;
				int spherePackingOptimizeIteratorNumber;

			public:
				OptimizeParameters() = default;
				virtual ~OptimizeParameters() {}
				OptimizeParameters(const OptimizeParameters& other) = default;
				OptimizeParameters(OptimizeParameters&& other) noexcept = default;
				OptimizeParameters& operator=(const OptimizeParameters& other) = default;
				OptimizeParameters& operator=(OptimizeParameters&& other) noexcept = default;
			};

			class DirectoryNames
			{
			public:
				std::string datFilesDirectoryName;
				std::string xsfFilesDirectoryName;

			public:
				DirectoryNames() = default;
				virtual ~DirectoryNames() {}
				DirectoryNames(const DirectoryNames& other) = default;
				DirectoryNames(DirectoryNames&& other) noexcept = default;
				DirectoryNames& operator=(const DirectoryNames& other) = default;
				DirectoryNames& operator=(DirectoryNames&& other) noexcept = default;
			};

			class FactoryAttribute
			{
			public:
				int maxTrialNumber;
				int thresholdTrialNumber;
				int createdNumber;
				int creatPlanNumber;

				double outputMinimumFillingFactor;
				double maxFillingFactor;

			public:
				FactoryAttribute() = default;
				virtual ~FactoryAttribute() {}
				FactoryAttribute(const FactoryAttribute& other) = default;
				FactoryAttribute(FactoryAttribute&& other) noexcept = default;
				FactoryAttribute& operator=(const FactoryAttribute& other) = default;
				FactoryAttribute& operator=(FactoryAttribute&& other) noexcept = default;
			};

			struct MutexCluster
			{
				std::mutex datMutex;
				std::mutex xsfMutex;
				std::mutex reportMutex;
			};

		public:
			PackingFactory(const BasisComposition& basisComposition, const PackingFactory::OptimizeParameters& optimizeParameters, const PackingFactory::DirectoryNames& directoryNames, PackingFactory::FactoryAttribute& factoryAttribute);
			virtual ~PackingFactory();

		public:
			PackingFactory(const PackingFactory& other) = default;
			PackingFactory(PackingFactory&& other) noexcept = default;
			PackingFactory& operator=(const PackingFactory& other) = default;
			PackingFactory& operator=(PackingFactory&& other) noexcept = default;

		public:
			void operator()(PackingFactory::MutexCluster& mutexCluster);

		private:
			void creations(std::vector<PackingStructure>& trialPackingStructures, RejectNumbers& rejectNumbers, PackingFactory::MutexCluster& mutexCluster);

		private:
			const BasisComposition m_basisComposition;

			const OptimizeParameters m_optimizeParameters;
			const DirectoryNames m_directoryNames;
			FactoryAttribute& m_factoryAttribute;

			const std::string m_version{ "VERSION 0.0" };
		};
	}
}


#endif // !RK_DNSPACK_PACKINGFACTORY_H
