#ifndef RK_DNSPACK_PACKINGSTRUCTURE_H
#define RK_DNSPACK_PACKINGSTRUCTURE_H

#include <vector>
#include <set>
#include <string>

#include "BravaisLattice.h"
#include "Sphere.h"
#include "BasisComposition.h"


namespace rk
{
	namespace dnspack
	{
		class PackingStructure
		{
		public:
			PackingStructure();
			PackingStructure(const BravaisLattice& bravaisLattice, const std::vector<Sphere>& basis);
			PackingStructure(const BravaisLattice& bravaisLattice, std::vector<Sphere>&& basis) noexcept;

			virtual ~PackingStructure();

		public:
			PackingStructure(const PackingStructure&) = default;
			PackingStructure(PackingStructure&&) noexcept = default;
			PackingStructure& operator=(const PackingStructure&) = default;
			PackingStructure& operator=(PackingStructure&&) noexcept = default;

			bool operator==(const PackingStructure& other) const;
			bool operator!=(const PackingStructure& other) const;

		public:
			const BravaisLattice& getBravaisLattice() const noexcept;
			BravaisLattice& getBravaisLattice() noexcept;
			const std::vector<Sphere>& getBasis() const noexcept;
			std::vector<Sphere>& getBasis() noexcept;

			std::size_t getSpheresNumber() const noexcept;
			std::size_t getSphereKindNumber() const noexcept;
			BasisComposition getBasisComposition() const;

			double getFillingFactor() const noexcept;
			bool overlapExistence(double& overlapVolume, double& minimumZ) const noexcept; //overlap݂ƂAtrueB݂ȂƂfalseő̐ς𐳂ԂB

		private:
			std::vector<Sphere> m_basis;
			BravaisLattice m_bravaisLattice;
		};
	}
}

inline bool rk::dnspack::PackingStructure::operator==(const PackingStructure& other) const
{
	return (m_basis == other.m_basis && m_bravaisLattice == other.m_bravaisLattice);
}

inline bool rk::dnspack::PackingStructure::operator!=(const PackingStructure& other) const
{
	return !(*this == other);
}

inline const rk::dnspack::BravaisLattice& rk::dnspack::PackingStructure::getBravaisLattice() const noexcept
{
	return m_bravaisLattice;
}

inline rk::dnspack::BravaisLattice& rk::dnspack::PackingStructure::getBravaisLattice() noexcept
{
	return m_bravaisLattice;
}

inline const std::vector<rk::dnspack::Sphere>& rk::dnspack::PackingStructure::getBasis() const noexcept
{
	return m_basis;
}

inline std::vector<rk::dnspack::Sphere>& rk::dnspack::PackingStructure::getBasis() noexcept
{
	return m_basis;
}

inline std::size_t rk::dnspack::PackingStructure::getSpheresNumber() const noexcept
{
	return m_basis.size();
}

inline std::size_t rk::dnspack::PackingStructure::getSphereKindNumber() const noexcept
{
	std::set<double> radii;

	for (const auto& sphere : m_basis)
	{
		radii.insert(sphere.getRadius());
	}

	return (radii.size());
}

#endif // !RK_DNSPACK_PACKINGSTRUCTURE_H
