#ifndef RK_DNSPACK_READINPUTFILE_H
#define RK_DNSPACK_READINPUTFILE_H

#include <string>
#include <vector>

#include "BasisComposition.h"

namespace rk
{
	namespace dnspack
	{
		class SphereInfo
		{
		public:
			SphereInfo();
			SphereInfo(const int ratio, const double minRadius, const double maxRadius, const double width);
			virtual ~SphereInfo();

		public:
			SphereInfo(const SphereInfo&) = default;
			SphereInfo(SphereInfo&&) noexcept = default;
			SphereInfo& operator=(const SphereInfo&) = default;
			SphereInfo& operator=(SphereInfo&&) noexcept = default;

		public:
			int compositionRatio;

			double minimumRadius;
			double maximumRadius;
			double widthOfRadiusTicks;
		};


		class ReadInputFile
		{
		public:
			explicit ReadInputFile(const std::string& fileName);
			virtual ~ReadInputFile();

		public:
			void initialize();

			const std::string& getOutputDirectoryName() const noexcept;

			const std::vector<SphereInfo>& getSphereInfo() const noexcept;

			int getMinimumSpheresNumber() const noexcept;
			int getMaximumSpheresNumber() const noexcept;

			int getThresholdTrialNumber() const noexcept;
			int getMaxTrialNumber() const noexcept;

			double getIntialExpandMaxChangeSize() const noexcept;
			int getIntialExpandMaxIteratorNumber() const noexcept;
			double getInitialArrangeMaxChangeSizeUpper() const noexcept;
			double getInitialArrangeMaxChangeSizeLower() const noexcept;
			int getIntialArrangeIteratorNumber() const noexcept;
			double getSpherePackingOptimizeMaxChangeSizeUpper() const noexcept;
			double getSpherePackingOptimizeMaxChangeSizeLower() const noexcept;
			double getSpherePackingOptimizeDecreaseFactor() const noexcept;
			int getSpherePackingOptimizeIteratorNumber() const noexcept;

			int getThreadNumber() const noexcept;

			double getOutputMinimumFillingFactor() const noexcept;

		private:
			std::string m_fileName;

			std::string m_outputDirectoryName{};

			std::vector<SphereInfo> m_sphereInfo{};

			int m_minimumSpheresNumber{ 0 };
			int m_maximumSpheresNumber{ 0 };

			int m_thresholdTrialNumber{ 0 };
			int m_maxTrialNumber{ 0 };

			double m_initialExpandMaxChangeSize{ 0.0 };
			int m_initialExpandMaxIteratorNumber{ 0 };
			double m_initialArrangeMaxChangeSizeUpper{ 0.0 };
			double m_initialArrangeMaxChangeSizeLower{ 0.0 };
			int m_initialArrangeIteratorNumber{ 0 };
			double m_spherePackingOptimizeMaxChangeSizeUpper{ 0.0 };
			double m_spherePackingOptimizeMaxChangeSizeLower{ 0.0 };
			double m_spherePackingOptimizeDecreaseFactor{ 0.0 };
			int m_spherepackingOptimizeIteratorNumber{ 0 };

			int m_threadNumber{ 0 };

			double m_outputMinimumFillingFactor{ 0.0 };

		private:
			ReadInputFile(const ReadInputFile&) = delete;
			ReadInputFile(ReadInputFile&&) noexcept = delete;
			ReadInputFile& operator=(const ReadInputFile&) = delete;
			ReadInputFile& operator=(ReadInputFile&&) noexcept = delete;
		};
	}
}


inline const std::string& rk::dnspack::ReadInputFile::getOutputDirectoryName() const noexcept
{
	return m_outputDirectoryName;
}

inline const std::vector<rk::dnspack::SphereInfo>& rk::dnspack::ReadInputFile::getSphereInfo() const noexcept
{
	return m_sphereInfo;
}

inline int rk::dnspack::ReadInputFile::getMinimumSpheresNumber() const noexcept
{
	return m_minimumSpheresNumber;
}

inline int rk::dnspack::ReadInputFile::getMaximumSpheresNumber() const noexcept
{
	return m_maximumSpheresNumber;
}

inline int rk::dnspack::ReadInputFile::getThresholdTrialNumber() const noexcept
{
	return m_thresholdTrialNumber;
}

inline int rk::dnspack::ReadInputFile::getMaxTrialNumber() const noexcept
{
	return m_maxTrialNumber;
}

inline double rk::dnspack::ReadInputFile::getIntialExpandMaxChangeSize() const noexcept
{
	return m_initialExpandMaxChangeSize;
}

inline int rk::dnspack::ReadInputFile::getIntialExpandMaxIteratorNumber() const noexcept
{
	return m_initialExpandMaxIteratorNumber;
}

inline double rk::dnspack::ReadInputFile::getInitialArrangeMaxChangeSizeUpper() const noexcept
{
	return m_initialArrangeMaxChangeSizeUpper;
}

inline double rk::dnspack::ReadInputFile::getInitialArrangeMaxChangeSizeLower() const noexcept
{
	return m_initialArrangeMaxChangeSizeLower;
}

inline int rk::dnspack::ReadInputFile::getIntialArrangeIteratorNumber() const noexcept
{
	return m_initialArrangeIteratorNumber;
}

inline double rk::dnspack::ReadInputFile::getSpherePackingOptimizeMaxChangeSizeUpper() const noexcept
{
	return m_spherePackingOptimizeMaxChangeSizeUpper;
}

inline double rk::dnspack::ReadInputFile::getSpherePackingOptimizeMaxChangeSizeLower() const noexcept
{
	return m_spherePackingOptimizeMaxChangeSizeLower;
}

inline double rk::dnspack::ReadInputFile::getSpherePackingOptimizeDecreaseFactor() const noexcept
{
	return m_spherePackingOptimizeDecreaseFactor;
}

inline int rk::dnspack::ReadInputFile::getSpherePackingOptimizeIteratorNumber() const noexcept
{
	return m_spherepackingOptimizeIteratorNumber;
}

inline int rk::dnspack::ReadInputFile::getThreadNumber() const noexcept
{
	return m_threadNumber;
}

inline double rk::dnspack::ReadInputFile::getOutputMinimumFillingFactor() const noexcept
{
	return m_outputMinimumFillingFactor;
}


#endif // !RK_DNSPACK_READINPUTFILE_H
