#ifndef RK_DNSPACK_REPORTCURRENTSITUATION_H
#define RK_DNSPACK_REPORTCURRENTSITUATION_H

#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <mutex>

namespace rk
{
	namespace dnspack
	{
		class ReportCurrentSituation
		{
		public:
			ReportCurrentSituation();
			virtual ~ReportCurrentSituation();

		public:
			static void setResultFileName(const std::string& resultFileName);

			void report(const std::string& message);
			void report(const std::string& message, std::mutex& outputMutex);
			void clear();

		private:
			static std::string m_fileName;

		private:
			ReportCurrentSituation(const ReportCurrentSituation& other) = delete;
			ReportCurrentSituation(ReportCurrentSituation&& other) noexcept = delete;
			ReportCurrentSituation& operator=(const ReportCurrentSituation& other) = delete;
			ReportCurrentSituation& operator=(ReportCurrentSituation&& other) noexcept = delete;
		};
	}
}


inline void rk::dnspack::ReportCurrentSituation::setResultFileName(const std::string& resultFileName)
{
	m_fileName = resultFileName;

	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open current situation file.");
	}

	outputFileStream.close();
}

inline void rk::dnspack::ReportCurrentSituation::clear()
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::out);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open current situation file.");
	}

	outputFileStream.close();
}

inline void rk::dnspack::ReportCurrentSituation::report(const std::string& message)
{
	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::app);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open current situation file.");
	}

	outputFileStream << message;
	outputFileStream.close();

	std::cout << message;
}

inline void rk::dnspack::ReportCurrentSituation::report(const std::string& message, std::mutex& outputMutex)
{
	std::lock_guard<std::mutex> lock{ outputMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(m_fileName, std::ios_base::app);

	if (!outputFileStream)
	{
		throw std::invalid_argument("couldn't open current situation file.");
	}

	outputFileStream << message;
	outputFileStream.close();

	std::cout << message;
}


#endif // !RK_DNSPACK_REPORTCURRENTSITUATION_H
