#ifndef RK_DNSPACK_SPHEREPACKINGOPTIMIZE_H
#define RK_DNSPACK_SPHEREPACKINGOPTIMIZE_H

#include <valarray>

#include "SphereLabel.h"
#include "PackingStructure.h"

namespace rk
{
	namespace dnspack
	{
		class SpherePackingOptimize
		{
		public:
			SpherePackingOptimize(PackingStructure& packingStructure, const int maxStepNumber, const double maxChangeSize, const double decreaseFactor);
			virtual ~SpherePackingOptimize();

		public:
			bool operator()();

		private:
			bool setNeighborSpheres();
			void calculateGradient();
			void optimize();

			double potential(const double z) const noexcept;
			double force(const double z) const noexcept;

		private:
			PackingStructure& m_packingStructure;

			const int m_spheresNumber;
			const int m_maxStepNumber;

			std::vector<std::pair<int, SphereLabel>> m_neighborSpheres{}; // baseIndex <= sphereIndex̏ꍇ̂
			std::valarray<double> m_gradientVector{};

			double m_maxChangeSize;
			double m_decreaseFactor;
			const double m_pressure{ 0.1 };

		private:
			SpherePackingOptimize(const SpherePackingOptimize&) = delete;
			SpherePackingOptimize(SpherePackingOptimize&&) noexcept = delete;
			SpherePackingOptimize& operator=(const SpherePackingOptimize&) = delete;
			SpherePackingOptimize& operator=(SpherePackingOptimize&&) noexcept = delete;
		};
	}
}


inline bool rk::dnspack::SpherePackingOptimize::operator()()
{
	int repNumber = m_maxStepNumber / 200;
	int repSurplus = m_maxStepNumber % 200;

	for (int rep = 0; rep < repNumber; ++rep)
	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < 200; ++step)
			{
				calculateGradient();
				optimize();

				m_maxChangeSize *= m_decreaseFactor;
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	{
		if (setNeighborSpheres())
		{
			for (int step = 0; step < repSurplus; ++step)
			{
				calculateGradient();
				optimize();

				m_maxChangeSize *= m_decreaseFactor;
			}

			if (m_packingStructure.getBravaisLattice().getVolume() < 0)
			{
				return false;
			}
		}

		else
		{
			return false;
		}
	}

	return true;
}

inline double rk::dnspack::SpherePackingOptimize::potential(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0 * z);
	}

	else
	{
		return (0);
	}
}

inline double rk::dnspack::SpherePackingOptimize::force(const double z) const noexcept
{
	if (z < 0)
	{
		return (-1.0);
	}

	else
	{
		return (0);
	}
}


#endif // !RK_DNSPACK_SPHEREPACKINGOPTIMIZE_H
