#include "ExploreDensestPackings.h"

#include <vector>
#include <thread>
#include <string>
#include <sstream>

#include "ReadInputFile.h"
#include "MakeDirectory.h"
#include "SetTrialBasisComposition.h"
#include "ExploreManager.h"

using namespace rk::dnspack;

ExploreDensestPackings::ExploreDensestPackings(rk::dnspack::IArgument& argument)
	: m_argument(argument)
{
}

ExploreDensestPackings::~ExploreDensestPackings()
{
}

int ExploreDensestPackings::execute()
{
	ExploreManager::ExploreParameters exploreParameters;

// ******************************************************************************************************************************************
	// Input Filẽf[^̓ǂݍ

	ReadInputFile readInputFile(m_argument.getInputFileName());
	readInputFile.initialize();

	exploreParameters.outputDirectoryName = readInputFile.getOutputDirectoryName();
	{
		if (!(exploreParameters.outputDirectoryName.back() == '/'))
		{
			throw std::invalid_argument("The last literature of Output.Directory.Name is not /");
		}
	}

	std::vector<SphereInfo> sphereInfo = readInputFile.getSphereInfo();

	int minimumSpheresNumber = readInputFile.getMinimumSpheresNumber();
	int maximumSpheresNumber = readInputFile.getMaximumSpheresNumber();

	exploreParameters.thresholdTrialNumber = readInputFile.getThresholdTrialNumber();
	exploreParameters.maxTrialNumber = readInputFile.getMaxTrialNumber();

	exploreParameters.initialExpandMaxChangeSize = readInputFile.getIntialExpandMaxChangeSize();
	exploreParameters.initialExpandMaxIteratorNumber = readInputFile.getIntialExpandMaxIteratorNumber();
	exploreParameters.initialArrangeMaxChangeSizeUpper = readInputFile.getInitialArrangeMaxChangeSizeUpper();
	exploreParameters.initialArrangeMaxChangeSizeLower = readInputFile.getInitialArrangeMaxChangeSizeLower();
	exploreParameters.initialArrangeIteratorNumber = readInputFile.getIntialArrangeIteratorNumber();
	exploreParameters.spherePackingOptimizeMaxChangeSizeUpper = readInputFile.getSpherePackingOptimizeMaxChangeSizeUpper();
	exploreParameters.spherePackingOptimizeMaxChangeSizeLower = readInputFile.getSpherePackingOptimizeMaxChangeSizeLower();
	exploreParameters.spherePackingOptimizeIteratorNumber = readInputFile.getSpherePackingOptimizeIteratorNumber();
	exploreParameters.spherePackingOptimizeDecreaseFactor = readInputFile.getSpherePackingOptimizeDecreaseFactor();

	exploreParameters.threadNumber = readInputFile.getThreadNumber();
	exploreParameters.outputMinimumFillingFactor = readInputFile.getOutputMinimumFillingFactor();

	// input filẽf[^̓ǂݍ
// ******************************************************************************************************************************************


	std::size_t sphereKindNumber = sphereInfo.size();

	if (6 < sphereKindNumber)
	{
		throw std::runtime_error("The kind of spheres is more than 6");
	}


// ******************************************************************************************************************************************
	// make output parent directory

	{
		std::string directoryName;

		for (const auto& info : sphereInfo)
		{
			directoryName += std::to_string(info.compositionRatio) + "-";
		}

		directoryName.pop_back();
		directoryName += '/';

		MakeDirectory makeDirectory(exploreParameters.outputDirectoryName, directoryName);
		makeDirectory();

		exploreParameters.outputDirectoryName += directoryName;
	}

	const std::string parentDirectoryPath = exploreParameters.outputDirectoryName;

	//make output parent directory
// ******************************************************************************************************************************************


	std::vector<BasisComposition> trialBasisComposition;

	SetTrialBasisComposition setTrialBasisComposition(sphereInfo, minimumSpheresNumber, maximumSpheresNumber);
	setTrialBasisComposition(trialBasisComposition);


	for (const auto& basisComposition : trialBasisComposition)
	{
		exploreParameters.outputDirectoryName = parentDirectoryPath;
		{
			std::string directoryName;

			for (const auto& number : basisComposition.getNumbers())
			{
				directoryName += std::to_string(number.second) + "-";
			}

			directoryName.pop_back();
			directoryName += '/';

			MakeDirectory makeDirectory(exploreParameters.outputDirectoryName, directoryName);
			makeDirectory();

			exploreParameters.outputDirectoryName += directoryName;
		}

		{
			std::string directoryName;

			for (const auto& radius : basisComposition.getRadii())
			{
				directoryName += std::to_string(radius) + "-";
			}

			directoryName.pop_back();
			directoryName += '/';

			MakeDirectory makeDirectory(exploreParameters.outputDirectoryName, directoryName);
			makeDirectory();

			exploreParameters.outputDirectoryName += directoryName;
		}

		ExploreManager exploreManager(basisComposition, exploreParameters);
		exploreManager.execute();
	}

	return 0;
}
