#include "MakeDirectory.h"

#include <stdexcept>

using namespace rk::dnspack;


#ifdef _MSC_VER


#include <filesystem>


MakeDirectory::MakeDirectory(const std::string& parentDirectoryPath, const std::string& childDirectoryName)
	: m_parentDirectoryPath{ parentDirectoryPath }
	, m_childDirectoryName{ childDirectoryName }
{
	if (!(m_parentDirectoryPath.back() == '/'))
	{
		throw std::invalid_argument("The last literature of parentDirectoryPath is not /");
	}

	if (!(m_childDirectoryName.back() == '/'))
	{
		throw std::invalid_argument("The last literature of childDirectoryPath is not /");
	}
}

MakeDirectory::~MakeDirectory()
{
}

bool MakeDirectory::operator()()
{
	std::filesystem::path parentDirectoryPath(m_parentDirectoryPath.c_str());

	if (!(std::filesystem::is_directory(parentDirectoryPath)))
	{
		std::string message;
		message += m_parentDirectoryPath;
		message += " is not a directory.";

		throw std::invalid_argument(message);
	}

	std::string childDirectoryPathName = m_parentDirectoryPath + m_childDirectoryName;
	std::filesystem::path childDirectoryPath(childDirectoryPathName.c_str());

	if ((std::filesystem::is_directory(childDirectoryPath)))
	{
		return false;
	}

	else
	{
		if (!(std::filesystem::create_directory(childDirectoryPath)))
		{
			std::string message;
			message += "couldn't make the ";
			message += childDirectoryPathName;

			throw std::invalid_argument(message);
		}
	}

	return true;
}


#else

#include <sys/stat.h>

MakeDirectory::MakeDirectory(const std::string& parentDirectoryPath, const std::string& childDirectoryName)
	: m_parentDirectoryPath{ parentDirectoryPath }
	, m_childDirectoryName{ childDirectoryName }
{
	if (!(m_parentDirectoryPath.back() == '/'))
	{
		throw std::invalid_argument("The last literature of parentDirectoryPath is not /");
	}

	if (!(m_childDirectoryName.back() == '/'))
	{
		throw std::invalid_argument("The last literature of childDirectoryName is not /");
	}
}

MakeDirectory::~MakeDirectory()
{
}

bool MakeDirectory::operator()()
{
	struct stat s;
	stat(m_parentDirectoryPath.c_str(), &s);

	if (!S_ISDIR(s.st_mode))
	{
		std::string message;
		message += m_parentDirectoryPath;
		message += " is not a directory.";

		throw std::invalid_argument(message);
	}

	std::string directoryPath = m_parentDirectoryPath + m_childDirectoryName;

	mkdir(directoryPath.c_str(), 0775);

	return true;
}


#endif // _MSC_VER
