#include "RandomGenerate.h"

#include <random>
#include <algorithm>
#include <numeric>
#include <iterator>

using namespace rk::dnspack;

RandomGenerate::RandomGenerate(const BasisComposition& basisComposition)
	: m_basisComposition(basisComposition)
	, m_spheresNumber(basisComposition.getSpheresNumber())
{
}

RandomGenerate::~RandomGenerate()
{
}


PackingStructure RandomGenerate::operator()()
{
	std::vector<double> radii(m_spheresNumber);
	std::vector<double>::iterator radiiIter = radii.begin();

	for (const auto& radiiAndNum : m_basisComposition.getNumbers())
	{
		double radius = radiiAndNum.first;
		int number = radiiAndNum.second;

		for (int num = 0; num < number; ++num)
		{
			*radiiIter = radius;
			++radiiIter;
		}
	}

	double minRadius = radii.front();
	radii.front() = radii.back();
	radii.back() = minRadius;

	std::random_device seed_gen;
	std::default_random_engine engine(seed_gen());


	double zLength = 0.0;
	for (const auto& radiiAndNum : m_basisComposition.getNumbers())
	{
		double radius = radiiAndNum.first;
		int number = radiiAndNum.second;

		zLength += radius * static_cast<double>(number);
	}
	zLength += 1.0;

	std::shuffle(std::next(radii.begin()), radii.end(), engine);
	radiiIter = radii.begin();

	BravaisLattice bravaisLattice;
	{
		std::uniform_real_distribution<double> angleDist(-1.0, 1.0);

		bravaisLattice[0].x = 2.0;

		bravaisLattice[1].x = angleDist(engine);
		bravaisLattice[1].y = 2.0;
	}

	std::vector<Sphere> basis;
	basis.reserve(m_spheresNumber);

	std::uniform_int_distribution<int> firstLayerDist(1, static_cast<int>(m_spheresNumber));
	std::size_t firstLayerSpheresNumber = static_cast<std::size_t>(firstLayerDist(engine));

	{
		zLength -= *radiiIter;

		Sphere sphere;
		sphere.setRadius(*radiiIter);
		++radiiIter;

		basis.push_back(sphere);
	}

	std::size_t sphereLabel = 1;

	for (; sphereLabel < firstLayerSpheresNumber; ++sphereLabel)
	{
		zLength -= *radiiIter;

		Sphere sphere;
		sphere.setRadius(*radiiIter);
		++radiiIter;

		std::uniform_real_distribution<double> coordinateDist(0.0, 1.0);

		sphere.getFractionalCoordinate().a = coordinateDist(engine);
		sphere.getFractionalCoordinate().b = coordinateDist(engine);

		basis.push_back(sphere);
	}

	std::uniform_real_distribution<double> ztiltingDist(-0.5, 0.5);

	bravaisLattice[2].x += ztiltingDist(engine) * 2.0;
	bravaisLattice[2] += scaling(ztiltingDist(engine), bravaisLattice[1]);
	bravaisLattice[2].z = zLength;

	for (double tempCoordinateZ = 0.0; sphereLabel < m_spheresNumber; ++sphereLabel)
	{
		tempCoordinateZ += *radiiIter;

		Sphere sphere;
		sphere.setRadius(*radiiIter);
		++radiiIter;

		std::uniform_real_distribution<double> coordinateDist(0.0, 1.0);

		sphere.getFractionalCoordinate().a = coordinateDist(engine);
		sphere.getFractionalCoordinate().b = coordinateDist(engine);
		sphere.getFractionalCoordinate().c = (tempCoordinateZ / zLength);

		basis.push_back(sphere);
	}

	return PackingStructure(bravaisLattice, std::move(basis));
}
