#include "ReadDatFile.h"

#include <fstream>
#include <stdexcept>
#include <sstream>
#include <regex>
#include <vector>

using namespace rk::dnspack;

ReadDatFile::ReadDatFile(const std::string& fileName)
	: m_fileName{ fileName }
{
}

ReadDatFile::~ReadDatFile()
{
}

PackingStructure ReadDatFile::operator()()
{
	std::string fileString;

	std::ifstream inputFileStream;
	inputFileStream.open(m_fileName, std::ios_base::in);
	
	if (!inputFileStream)
	{
		std::string message{ "could not open " };
		message += m_fileName + '.';

		throw std::runtime_error(message);
	}
	
	for (std::string stringLine; std::getline(inputFileStream, stringLine);)
	{
		fileString += stringLine + "\n";
	}
	inputFileStream.close();


	std::map<std::string, double> nameAndRadii;
	{
		std::size_t initialPos = fileString.find("<Definition.of.Sphere.Species");
		std::size_t endPos = fileString.find("Definition.of.Sphere.Species>");

		if (initialPos != std::string::npos && endPos != std::string::npos)
		{
			std::size_t tempPos = fileString.find_first_of("\n", initialPos);
			std::string spheresString{ fileString, (tempPos + 1), (endPos - tempPos - 1) };
			std::stringstream spheresSStream{ spheresString };

			std::regex pat("[\\s]*([\\w]+)[\\s]+([\\d.]+)[\\s]*");

			for (std::string line; std::getline(spheresSStream, line);)
			{
				std::smatch compositionSm;

				if (std::regex_match(line, compositionSm, pat))
				{
					nameAndRadii.emplace(compositionSm.str(1), std::stod(compositionSm.str(2)));
				}

				else
				{
					throw std::runtime_error("couldn't get the Definition.of.Sphere.Species.");
				}
			}
		}

		else
		{
			throw std::runtime_error("couldn't get the Definition.of.Sphere.Species.");
		}
	}

	{
		std::regex pat("[\\s]*Atoms.SpeciesAndCoordinates.Unit[\\s]+([\\w]+)[\\s]*");
		std::smatch numSm;

		if (std::regex_search(fileString, numSm, pat))
		{
			if (numSm.str(1) != "FRAC")
			{
				throw std::runtime_error("Coordinates have to be written in fractional coordinates.");
			}
		}

		else
		{
			throw std::runtime_error("couldn't get Atoms.SpeciesAndCoordinates.Unit.");
		}
	}
	
	std::vector<Sphere> basis;
	{
		std::size_t initialPos = fileString.find("<Atoms.SpeciesAndCoordinates");
		std::size_t endPos = fileString.find("Atoms.SpeciesAndCoordinates>");

		if (initialPos != std::string::npos && endPos != std::string::npos)
		{
			std::size_t tempPos = fileString.find_first_of("\n", initialPos);
			std::string spheresString{ fileString, (tempPos + 1), (endPos - tempPos - 1) };
			std::stringstream spheresSStream{ spheresString };

			std::regex pat("[\\s]*[\\d]+[\\s]+([\\w]+)[\\s]+([\\d.-]+)[\\s]+([\\d.-]+)[\\s]+([\\d.-]+)[\\s]*");

			for (std::string line; std::getline(spheresSStream, line);)
			{
				std::smatch coordinateSm;

				if (std::regex_match(line, coordinateSm, pat))
				{
					Sphere sphere;
					sphere.setRadius(nameAndRadii.at(coordinateSm.str(1)));

					sphere.getFractionalCoordinate().a = std::stod(coordinateSm.str(2));
					sphere.getFractionalCoordinate().b = std::stod(coordinateSm.str(3));
					sphere.getFractionalCoordinate().c = std::stod(coordinateSm.str(4));

					basis.push_back(std::move(sphere));
				}

				else
				{
					throw std::runtime_error("couldn't get the Atoms.SpeciesAndCoordinates.");
				}
			}
		}

		else
		{
			throw std::runtime_error("couldn't get the Atoms.SpeciesAndCoordinates.");
		}
	}

	BravaisLattice bravaisLattice;
	{
		std::size_t initialPos = fileString.find("<Atoms.UnitVectors");
		std::size_t endPos = fileString.find("Atoms.UnitVectors>");

		if (initialPos != std::string::npos && endPos != std::string::npos)
		{
			std::size_t tempPos = fileString.find_first_of("\n", initialPos);
			std::string spheresString{ fileString, (tempPos + 1), (endPos - tempPos - 1) };
			std::stringstream spheresSStream{ spheresString };

			std::regex pat("[\\s]*([\\d.-]+)[\\s]+([\\d.-]+)[\\s]+([\\d.-]+)[\\s]*");

			int label = 0;

			for (std::string line; std::getline(spheresSStream, line);)
			{
				std::smatch latticeVectorSm;

				if (std::regex_match(line, latticeVectorSm, pat))
				{
					bravaisLattice[label].x = std::stod(latticeVectorSm.str(1));
					bravaisLattice[label].y = std::stod(latticeVectorSm.str(2));
					bravaisLattice[label].z = std::stod(latticeVectorSm.str(3));

					++label;
				}

				else
				{
					throw std::runtime_error("couldn't get the Atoms.UnitVectors.");
				}
			}
		}

		else
		{
			throw std::runtime_error("couldn't get the Atoms.UnitVectors.");
		}
	}

	return PackingStructure(bravaisLattice, std::move(basis));
}
